#!/usr/bin/env python3

# Copyright 2023 Battelle Energy Alliance, LLC

# python std library imports
import pkg_resources

# third party imports
import click

# package imports
from navv.commands import generate, launch
from navv.message_handler import info_msg
from navv._version import __version__


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
HEADER = f"NAVV: Network Architecture Verification and Validation {__version__}"
DATA_PATH = pkg_resources.resource_filename("navv", "data/")


@click.group(context_settings=CONTEXT_SETTINGS, invoke_without_command=True)
@click.version_option(__version__)
@click.pass_context
def cli(ctx):
    """Network Architecture Verification and Validation."""
    if ctx.invoked_subcommand is None:
        info_msg(HEADER)
        print(ctx.command.get_help(ctx))
    pass


def main():
    """Main function for performing zeek-cut commands and sorting the output"""

    cli.add_command(generate)
    cli.add_command(launch)
    cli()


if __name__ == "__main__":
    main()



import json
import os
import pandas as pd

from navv.utilities import get_mac_vendor, timeit
from navv.validators import is_ipv4_address, is_ipv6_address


MAC_VENDORS_JSON_FILE = os.path.abspath(__file__ + "/../" + "data/mac-vendors.json")


def get_zeek_df(zeek_data: list, dns_data: dict):
    """Return a pandas dataframe of the conn.log data with its dns data."""
    zeek_data = [row.split("\t") for row in zeek_data]
    # Insert dns data to zeek data
    for row in zeek_data:
        row.insert(1, dns_data.get(row[0], ""))
        row.insert(3, dns_data.get(row[2], ""))

    return pd.DataFrame(
        zeek_data,
        columns=[
            "src_ip",
            "src_hostname",
            "dst_ip",
            "dst_hostname",
            "port",
            "proto",
            "conn",
            "src_mac",
            "dst_mac",
        ],
    )


@timeit
def get_inventory_report_df(zeek_df: pd.DataFrame):
    """Return a pandas dataframe of the inventory report data."""
    zeek_df["port_and_proto"] = zeek_df["port"] + "/" + zeek_df["proto"]

    zeek_df["src_ipv4"] = zeek_df["src_ip"].apply(
        lambda ip: ip if is_ipv4_address(ip) else None
    )
    zeek_df["src_ipv6"] = zeek_df["src_ip"].apply(
        lambda ip: ip if is_ipv6_address(ip) else None
    )

    zeek_df["dst_ipv4"] = zeek_df["dst_ip"].apply(
        lambda ip: ip if is_ipv4_address(ip) else None
    )
    zeek_df["dst_ipv6"] = zeek_df["dst_ip"].apply(
        lambda ip: ip if is_ipv6_address(ip) else None
    )

    src_df = zeek_df[
        [
            "src_mac",
            "src_ipv4",
            "src_hostname",
            "src_ipv6",
            "dst_ipv4",
            "dst_hostname",
            "dst_ipv6",
            "port_and_proto",
        ]
    ].reset_index(drop=True)
    src_df["mac"] = src_df["src_mac"]

    dst_df = zeek_df[
        [
            "dst_mac",
            "src_ipv4",
            "src_hostname",
            "src_ipv6",
            "dst_ipv4",
            "dst_hostname",
            "dst_ipv6",
            "port_and_proto",
        ]
    ].reset_index(drop=True)
    dst_df["mac"] = dst_df["dst_mac"]

    df = (
        pd.concat([src_df, dst_df])
        .reset_index(drop=True)
        .drop(columns=["src_mac", "dst_mac"])
        .drop_duplicates(
            subset=[
                "src_ipv4",
                "src_hostname",
                "src_ipv6",
                "dst_ipv4",
                "dst_hostname",
                "dst_ipv6",
                "port_and_proto",
            ]
        )
    )

    grouped_df = (
        df.groupby("mac", as_index=False)
        .agg(
            {
                "src_ipv4": list,
                "src_hostname": list,
                "src_ipv6": list,
                "dst_ipv4": list,
                "dst_hostname": list,
                "dst_ipv6": list,
                "port_and_proto": list,
            }
        )
        .reset_index()
    )

    mac_vendors = {}
    with open(MAC_VENDORS_JSON_FILE) as f:
        mac_vendors = json.load(f)
    grouped_df["vendor"] = grouped_df["mac"].apply(
        lambda mac: get_mac_vendor(mac_vendors, mac)
    )
    grouped_df["ipv4"] = (grouped_df["src_ipv4"] + grouped_df["dst_ipv4"]).apply(
        lambda ip: list(set(ip))
    )
    grouped_df["ipv6"] = (grouped_df["src_ipv6"] + grouped_df["dst_ipv6"]).apply(
        lambda ip: list(set(ip))
    )
    grouped_df["hostname"] = (
        grouped_df["src_hostname"] + grouped_df["dst_hostname"]
    ).apply(lambda hostname: list(set(hostname)))

    grouped_df.drop(
        columns=[
            "src_ipv4",
            "src_hostname",
            "src_ipv6",
            "dst_ipv4",
            "dst_hostname",
            "dst_ipv6",
        ],
        inplace=True,
    )

    return grouped_df


@timeit
def get_snmp_df(zeek_data: list):
    """Return a pandas dataframe of the snmp.log data."""
    zeek_data = [row.split("\t") for row in zeek_data]
    return pd.DataFrame(
        zeek_data,
        columns=[
            "src_ip",
            "src_port",
            "dst_ip",
            "dst_port",
            "version",
            "community",
        ],
    )



import json
import os
from subprocess import Popen, PIPE, STDOUT, check_call

from navv.message_handler import error_msg
from navv.utilities import pushd, timeit, trim_dns_data


@timeit
def get_conn_data(zeek_logs):
    """Return a list of Zeek conn.log data."""
    return (
        perform_zeekcut(
            fields=[
                "id.orig_h",
                "id.resp_h",
                "id.resp_p",
                "proto",
                "conn_state",
                "orig_l2_addr",
                "resp_l2_addr",
            ],
            log_file=os.path.join(zeek_logs, "conn.log"),
        )
        .decode("utf-8")
        .split("\n")[:-1]
    )


@timeit
def get_dns_data(customer_name, output_dir, zeek_logs):
    """Get DNS data from zeek logs or from a json file if it exists"""
    json_path = os.path.join(output_dir, f"{customer_name}_dns_data.json")
    if os.path.exists(json_path):
        with open(json_path, "rb") as json_file:
            return json.load(json_file)

    dns_data = perform_zeekcut(
        fields=["query", "answers", "qtype", "rcode_name"],
        log_file=os.path.join(zeek_logs, "dns.log"),
    )
    return trim_dns_data(dns_data)


@timeit
def get_snmp_data(zeek_logs):
    """Get SNMP data from zeek logs or from a json file if it exists"""
    return (
        perform_zeekcut(
            fields=[
                "id.orig_h",
                "id.orig_p",
                "id.resp_h",
                "id.resp_p",
                "version",
                "community",
            ],
            log_file=os.path.join(zeek_logs, "snmp.log"),
        )
        .decode("utf-8")
        .split("\n")[:-1]
    )


def perform_zeekcut(fields, log_file):
    """Perform the call to zeek-cut with the identified fields on the specified log file"""
    try:
        with open(log_file, "rb") as f:
            zeekcut = Popen(
                ["zeek-cut"] + fields, stdout=PIPE, stdin=PIPE, stderr=STDOUT
            )
            return zeekcut.communicate(input=f.read())[0]
    except OSError as e:
        # probably "file does not exist"
        return b""


@timeit
def run_zeek(pcap_path, zeek_logs_path, **kwargs):
    with pushd(zeek_logs_path):
        # can we add Site::local_nets to the zeek call here?
        try:
            check_call(["zeek", "-C", "-r", pcap_path, "local.zeek"])
        except Exception as e:
            error_msg(e)



import logging
import os

from flask import Flask, render_template, request, send_file
from navv.gui.bll import generate

from navv.gui.utils import get_pcap_file


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
app = Flask(__name__)


@app.route("/")
def index():
    """Home page."""
    pcap_file, pcap_msg, pcap_msg_color = get_pcap_file()

    return render_template(
        "home.html",
        pcap_file=pcap_file,
        pcap_msg=pcap_msg,
        pcap_msg_color=pcap_msg_color,
    )


@app.route("/new-analysis")
def new_analysis():
    pcap_file, pcap_msg, pcap_msg_color = get_pcap_file()

    return render_template(
        "create_new.html",
        pcap_file=pcap_file,
        pcap_msg=pcap_msg,
        pcap_msg_color=pcap_msg_color,
    )


@app.route("/existing-analysis")
def existing_analysis():
    pcap_file, pcap_msg, pcap_msg_color = get_pcap_file()

    return render_template(
        "update_existing.html",
        pcap_file=pcap_file,
        pcap_msg=pcap_msg,
        pcap_msg_color=pcap_msg_color,
    )


@app.route("/download", methods=["POST"])
def download():
    """Download the network analysis excel file."""
    # Get customer name
    customer_name = request.form["customername"]
    filename = f"{customer_name}_network_analysis.xlsx"

    # Set output directory
    output_dir = os.path.join(os.getcwd(), "_tmp")
    if not os.path.isdir(output_dir):
        os.mkdir(output_dir)

    # set excel file
    excel_file = request.files.get("spreadsheet")
    if excel_file and excel_file.filename:
        excel_file.save(os.path.join(output_dir, excel_file.filename))

    # Get pcap file and Zeek logs if available
    pcap_file = request.files.get("pcapfile")
    if pcap_file and pcap_file.filename:
        pcap_file.save(os.path.join(output_dir, pcap_file.filename))

    zeek_logs = request.files.get("zeeklogs")
    if zeek_logs and zeek_logs.filename:
        zeek_logs.save(os.path.join(output_dir, zeek_logs.filename))

    memfile = generate(
        customer_name, output_dir, pcap_file, zeek_logs.filename, excel_file
    )

    return send_file(memfile, download_name=filename, as_attachment=True)



import os
import logging

logger = logging.getLogger(__name__)


def get_pcap_file() -> tuple[str, str, str]:
    """Get the pcap file from the current working directory."""
    pcap_files = set()
    for file in os.listdir(os.getcwd()):
        if file.endswith(".pcap"):
            logger.info(f"Found pcap file: {file} in {os.getcwd()}")
            pcap_files.add(file)

    # No PCAP file found
    if len(pcap_files) == 0:
        logger.error(f"Could not find a pcap file in {os.getcwd()}")
        return (
            "",
            "Could not find a pcap file in the current directory. Please upload.",
            "warning",
        )

    # Multiple PCAP files found
    if len(pcap_files) > 1:
        logger.error(f"Found multiple pcap files, please remove all but one.")
        return "", "Found multiple pcap files. Please remove all but one.", "danger"

    filename = pcap_files.pop()
    return filename, f"{filename} detected and uploaded successfully.", "success"



"""CLI Commands."""
import os
import webbrowser


# Third-Party Libraries
import click

# cisagov Libraries
from navv.gui.app import app
from navv.bll import get_inventory_report_df, get_snmp_df, get_zeek_df
from navv.message_handler import success_msg, warning_msg
from navv.spreadsheet_tools import (
    auto_adjust_width,
    create_analysis_array,
    get_inventory_data,
    get_package_data,
    get_segments_data,
    get_workbook,
    perform_analysis,
    write_conn_states_sheet,
    write_externals_sheet,
    write_inventory_report_sheet,
    write_snmp_sheet,
    write_stats_sheet,
    write_unknown_internals_sheet,
)
from navv.zeek import (
    get_conn_data,
    get_dns_data,
    get_snmp_data,
    run_zeek,
    perform_zeekcut,
)
from navv.utilities import pushd


@click.command("generate")
@click.option(
    "-o",
    "--output-dir",
    required=False,
    help="Directory to place resultant analysis files in. Defaults to current working directory.",
    type=str,
)
@click.option(
    "-p",
    "--pcap",
    required=False,
    help="Path to pcap file. NAVV requires zeek logs or pcap. If used, zeek will run on pcap to create new logs.",
    type=str,
)
@click.option(
    "-z",
    "--zeek-logs",
    required=False,
    help="Path to store or contain zeek log files. Defaults to current working directory.",
    type=str,
)
@click.argument("customer_name")
def generate(customer_name, output_dir, pcap, zeek_logs):
    """Generate excel sheet."""
    with pushd(output_dir):
        pass
    file_name = os.path.join(output_dir, customer_name + "_network_analysis.xlsx")

    wb = get_workbook(file_name)

    services, conn_states = get_package_data()
    timer_data = dict()
    segments = get_segments_data(wb["Segments"])
    inventory = get_inventory_data(wb["Inventory Input"])

    if pcap:
        run_zeek(os.path.abspath(pcap), zeek_logs, timer=timer_data)
    else:
        timer_data["run_zeek"] = "NOT RAN"

    # Get zeek data from conn.log, dns.log and snmp.log
    zeek_data = get_conn_data(zeek_logs)
    snmp_data = get_snmp_data(zeek_logs)
    dns_filtered = get_dns_data(customer_name, output_dir, zeek_logs)

    # Get dns data for resolution
    json_path = os.path.join(output_dir, f"{customer_name}_dns_data.json")

    # Get zeek dataframes
    zeek_df = get_zeek_df(zeek_data, dns_filtered)
    snmp_df = get_snmp_df(snmp_data)

    # Get inventory report dataframe
    inventory_df = get_inventory_report_df(zeek_df)

    # Turn zeekcut data into rows for spreadsheet
    rows = create_analysis_array(zeek_data, timer=timer_data)

    ext_IPs = set()
    unk_int_IPs = set()
    perform_analysis(
        wb,
        rows,
        services,
        conn_states,
        inventory,
        segments,
        dns_filtered,
        json_path,
        ext_IPs,
        unk_int_IPs,
        timer=timer_data,
    )

    write_inventory_report_sheet(inventory_df, wb)

    write_externals_sheet(ext_IPs, wb)

    write_unknown_internals_sheet(unk_int_IPs, wb)

    write_snmp_sheet(snmp_df, wb)

    auto_adjust_width(wb["Analysis"])

    times = (
        perform_zeekcut(fields=["ts"], log_file=os.path.join(zeek_logs, "conn.log"))
        .decode("utf-8")
        .split("\n")[:-1]
    )
    forward = sorted(times)
    start = float(forward[0])
    end = float(forward[len(forward) - 1])
    cap_time = end - start
    timer_data[
        "Length of Capture time"
    ] = "{} day(s) {} hour(s) {} minutes {} seconds".format(
        int(cap_time / 86400),
        int(cap_time % 86400 / 3600),
        int(cap_time % 3600 / 60),
        int(cap_time % 60),
    )
    write_stats_sheet(wb, timer_data)
    write_conn_states_sheet(conn_states, wb)

    wb.save(file_name)

    if pcap:
        success_msg(f"Successfully created file: {file_name}")


@click.command("launch")
def launch():
    """Launch the NAVV GUI."""
    port = 5000
    warning_msg("Launching GUI in browser...")
    webbrowser.open(f"http://127.0.0.1:{port}/")
    app.run(port=port)
